{ version: 1.0.1.1
	Programmer: Dinko Miljak - dinmi@flashmail.com
	Edited by Anthony Frazier - afrazier@infonline.net
	This class encapsulates CGI function neccesary to write normal cgi scripts
	This class is developed so programmers can create small CGI executables,
	Typically around 50 K
	Some function names are taken from CGIC library (FormInteger, FormString,
	FormFloat)
	Basic idea how to make this class was taken from Robert Wuttke

	Update 2000-10-30: Function LocalTime to GMT Time - important for cookie handling
	Update 2000-11-22: Ted Toraasen <ted@biip.no> -> GetValue function
	Update 2000-11-23: GetCookie function is updated
	Update 2001-03-26: Netscape 6 problem - Netscape 6 shows nothing when script
										 is executed. Problem is ReturnHeader function.
										 Thanks to Ben Hay this problem is solved
	Update 2001-05-21: Code cleanup.  Uses writeln, improved string handling.
										 Update by Anthony Frazier
	Update 2001-05-22: Big stupid bugfix in GetValue.  By Anthony Frazier.
}

unit cgilib;

interface

uses Windows, SysUtils;

const CGILIB_VERSION: string = '1.0.1.1';

type
	TCGI = class
	private
		EB : Pointer; // Environment Block Pointer
		FAUTH_TYPE: string;
		FCONTENT_LENGTH: string;
		FCONTENT_TYPE: string;
		FGATEWAY_INTERFACE: string;
		FPATH_INFO: string;
		FPATH_TRANSLATED: string;
		FREMOTE_ADDR: string;
		FREMOTE_HOST: string;
		FREMOTE_IDENT: string;
		FREMOTE_USER: string;
		FREQUEST_METHOD: string;
		FSCRIPT_NAME: string;
		FSERVER_NAME: string;
		FSERVER_PORT: string;
		FSERVER_PROTOCOL: string;
		FSERVER_SOFTWARE: string;
		FQUERY_STRING: string;
		FHTTP_ACCEPT: string;
		FHTTP_USER_AGENT:string;
		FHTTP_REFERER: string;
		FHTTP_COOKIE: string;
		FCOOKIES: string;
	protected
		procedure GetEnvironments; // Get the default Environment Variables
	public
		constructor Create;
		destructor Destroy; override;

		property AUTH_TYPE         : string read FAUTH_TYPE;
		property CONTENT_LENGTH    : string read FCONTENT_LENGTH;
		property CONTENT_TYPE      : string read FCONTENT_TYPE;
		property GATEWAY_INTERFACE : string read FGATEWAY_INTERFACE;
		property PATH_INFO         : string read FPATH_INFO;
		property PATH_TRANSLATED   : string read FPATH_TRANSLATED;
		property REMOTE_ADDR       : string read FREMOTE_ADDR;
		property REMOTE_HOST       : string read FREMOTE_HOST;
		property REMOTE_IDENT      : string read FREMOTE_IDENT;
		property REMOTE_USER       : string read FREMOTE_USER;
		property REQUEST_METHOD    : string read FREQUEST_METHOD;
		property SCRIPT_NAME       : string read FSCRIPT_NAME;
		property SERVER_NAME       : string read FSERVER_NAME;
		property SERVER_PORT       : string read FSERVER_PORT;
		property SERVER_PROTOCOL   : string read FSERVER_PROTOCOL;
		property SERVER_SOFTWARE   : string read FSERVER_SOFTWARE;
		property QUERY_STRING      : string read FQUERY_STRING;
		property HTTP_ACCEPT       : string read FHTTP_ACCEPT;
		property HTTP_USER_AGENT   : string read FHTTP_USER_AGENT;
		property HTTP_REFERER      : string read FHTTP_REFERER;
		// This has the cookies from the browser
		property HTTP_COOKIE       : string read FHTTP_COOKIE;
		// This is the newly formed Cookie string
		property COOKIES           : string read FCOOKIES;

		function HTTPDecode(const AStr: string): string; // Borland routine
		function HTTPEncode(const AStr: string): string; // Borland routine
		function XMLEncode(var InStr: string): string;

		function GetEnvVariable(Name : string) : string;

		// These functions return true if VariableName is found
		function GetValue(VarName: string; var VarValue: string): boolean;
		function FormInteger(VarName: string; var RetValue: integer; DefValue: integer): boolean;
		function FormFloat(VarName: string; var RetValue: double; DefValue: double): boolean;
		function FormString(VarName: string; var RetValue: string; DefValue: string): boolean;

		// These functions return HTTP header
		procedure ReturnHeaderExpire(ContentType: string);
		procedure ReturnHeader(ContentType: string; Expire: TDateTime);

		procedure Redirect(URL: string);

		// This function returns Environment variables to browser for debugging
		procedure ReturnVariables;

		// XML Functions - every function returns <node>value</node>LN_BR
		function WriteOneNodeS(NodeName, NodeValue: string): string;
		function WriteOneNode(NodeName, NodeValue: string): string;

		// These functions return Month and DayOfWeek in string format - Borland routines
		function DayOfWeekStr(DateTime: TDateTime): string;
		function MonthStr(DateTime: TDateTime): string;

		// This function resets CookieString (which will be sent to browser as '')
		procedure ResetCookieString;

		// This function properly forms one cookie string
		procedure AddCookie(CookieName: string; CookieValue: string;
			Expire: TDateTime; PathName: string; DomainName: string; Secure: boolean);

		// This function finds the value of cookie CookieName and returns it's
		// value in CookieValue.
		// This function returns true if CookieName is found.
		function GetCookie(CookieName: string; var CookieValue: string): boolean;

		function LocalToGMT(InTime: TDateTime): TDateTime;
	end;

implementation

const
	LN_BR: string = #13 + #10;
	DateFormat = '"%s", dd "%s" yyyy hh:mm:ss';
	DaysOfWeek: array[1..7] of string = (
		'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
	Months: array[1..12] of string = (
		'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
		'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');
	NoConv = ['A'..'Z', 'a'..'z', '*', '@', '.', '_', '-',
						'0'..'9', '$', '!', '''', '(', ')'];

  { Following declarations are needed because Delphi 2
    does not have this declarations inside Windows.pas unit }
  TIME_ZONE_ID_INVALID = DWORD($FFFFFFFF);
  TIME_ZONE_ID_UNKNOWN  = 0;
  TIME_ZONE_ID_STANDARD = 1;
  TIME_ZONE_ID_DAYLIGHT = 2;

constructor TCGI.Create;
	begin
		EB := GetEnvironmentStrings; // Open the environment block
		if (EB = nil) then begin
			writeln('Environment block error - something is very wrong.');
			Halt;
		end;
		GetEnvironments;
		{ Even though TextOut is no longer used, keep this call around for
			compatability. }
		Rewrite(Output);
	end;

destructor TCGI.Destroy;
	begin
		FreeEnvironmentStrings(EB); { Close the environment block }
	end;

{ Classic Borland function for HTTP Decoding }
function TCGI.HTTPDecode(const AStr: string): string;
	var
		Sp, Rp, Cp: PChar;

	begin
		SetLength(Result, Length(AStr));
		Sp := PChar(AStr);
		Rp := PChar(Result);
		while (Sp^ <> #0) do begin
			case Sp^ of
				'+': Rp^ := ' ';
				'%': begin
					inc(Sp);
					if (Sp^ = '%') then
						Rp^ := '%'
					else begin
						Cp := Sp;
						Inc(Sp);
						Rp^ := Chr(StrToInt(Format('$%s%s',[Cp^, Sp^])));
					end;
				end;
			else
				Rp^ := Sp^;
			end;
			Inc(Rp);
			Inc(Sp);
		end;
		SetLength(Result, Rp - PChar(Result));
	end;

{ Classic Borland function for HTTP Encoding }
function TCGI.HTTPEncode(const AStr: string): string;
	var
		Sp, Rp: PChar;

	begin
		SetLength(Result, Length(AStr) * 3);
		Sp := PChar(AStr);
		Rp := PChar(Result);
		while (Sp^ <> #0) do begin
			if (Sp^ in NoConv) then
				Rp^ := Sp^
			else if (Sp^ = ' ') then
				Rp^ := '+'
			else begin
				FormatBuf(Rp^, 3, '%%%.2x', 6, [Ord(Sp^)]);
				Inc(Rp,2);
			end;
			Inc(Rp);
			Inc(Sp);
		end;
		SetLength(Result, Rp - PChar(Result));
	end;

{ My function for XML Encoding }
function TCGI.XMLEncode(var InStr: string): string;
	var
		i: integer;

	begin
		Result := '';
		for i := 1 to Length(inStr) do
			if (inStr[i] in NoConv) then
				Result := Result + inStr[i]
			else begin
				case inStr[i] of
					'<' : Result := Result + '&lt;';
					'>' : Result := Result + '&gt;';
					'&' : Result := Result + '&amp;';
					'''': Result := Result + '&apos;';
	//        '"' : Result := Result + '&qout;';
				else
					Result := Result + Format('&#x%2.2X;', [Ord(inStr[i])]);
				end;
			end;
	end;

{ This function returns one environment variable }
function TCGI.GetEnvVariable(Name : string) : string;
	var
		Buf: PChar;
		BufSize: Integer;

	begin
		Result := '';
		{ The GetMem/FreeMem commands here may be removed, and the 1 replaced with a
			0 in the GetEnvironmentVariable() call, but doing it this way suppresses a
			warning from Delphi. }
		GetMem(Buf, 1);
		BufSize := GetEnvironmentVariable(PChar(Name), Buf, 1);
		FreeMem(Buf);
		if (BufSize > 0) then begin
			GetMem(Buf, BufSize + 1);
			GetEnvironmentVariable(PChar(Name), Buf, BufSize);
			SetString(Result, Buf, StrLen(Buf));
			FreeMem(Buf);
		end;
	end;

{ First step is to get all environment variables }
procedure TCGI.GetEnvironments;
	var
		i: integer;
		ch: Char;
		TmpLen: integer;

	begin
		FAUTH_TYPE := GetEnvVariable('AUTH_TYPE');
		FCONTENT_TYPE := GetEnvVariable('CONTENT_TYPE');
		FCONTENT_LENGTH := GetEnvVariable('CONTENT_LENGTH');
		FGATEWAY_INTERFACE := GetEnvVariable('GATEWAY_INTERFACE');
		FPATH_INFO := GetEnvVariable('PATH_INFO');
		FPATH_TRANSLATED := GetEnvVariable('PATH_TRANSLATED');
		FREMOTE_ADDR := GetEnvVariable('REMOTE_ADDR');
		FREMOTE_HOST := GetEnvVariable('REMOTE_HOST');
		FREMOTE_IDENT := GetEnvVariable('REMOTE_IDENT');
		FREMOTE_USER := GetEnvVariable('REMOTE_USER');
		FREQUEST_METHOD := GetEnvVariable('REQUEST_METHOD');
		FSCRIPT_NAME := GetEnvVariable('SCRIPT_NAME');
		FSERVER_NAME := GetEnvVariable('SERVER_NAME');
		FSERVER_PORT := GetEnvVariable('SERVER_PORT');
		FSERVER_PROTOCOL := GetEnvVariable('SERVER_PROTOCOL');
		FSERVER_SOFTWARE := GetEnvVariable('SERVER_SOFTWARE');
		FHTTP_ACCEPT := GetEnvVariable('HTTP_ACCEPT');
		FHTTP_USER_AGENT := GetEnvVariable('HTTP_USER_AGENT');
		FHTTP_REFERER := GetEnvVariable('HTTP_REFERER');
		FHTTP_COOKIE := GetEnvVariable('HTTP_COOKIE');

		if (CompareText(FREQUEST_METHOD, 'GET') = 0) then begin
			FQUERY_STRING := GetEnvVariable('QUERY_STRING');
			FCONTENT_LENGTH := IntToStr(Length(FQUERY_STRING));
		end else if (CompareText(FREQUEST_METHOD,'POST') = 0) then begin
			FQUERY_STRING := '';
			TmpLen := StrToIntDef(FCONTENT_LENGTH, 0);
			if (TmpLen > 0) then begin
				// We have to read the exact amount of characters
				// as there may be more data on the standard input
				for i := 1 to TmpLen do begin
					Read(ch);
					FQUERY_STRING := FQUERY_STRING + ch;
				end;
				FQUERY_STRING := FQUERY_STRING + #0;
			end;
		end else // UNKNOWN Method
			FQUERY_STRING := '';
	end;

{ Get one variable value
	VarName -> variable name to look for
	VarValue -> in this variable is saved result and returned to calling function
	Result is true if VarName is founded }
function TCGI.GetValue(VarName: string; var VarValue: string): boolean;
	var
		TmpPos: integer;
		TmpVal: string;

	begin
		// this is changed thanks to Ted Toraasen <ted@biip.no>
		// Prior version source:
		//    VarPos := Pos(VarName + '=', FQUERY_STRING);
		TmpPos := Pos('&' + VarName + '=', '&' + FQUERY_STRING);
		Result := TmpPos > 0;
		if Result then begin
			{ Since we already know WHERE the value is in the Query string, we can
				skip over it when trying to find that value. }
			TmpVal := Copy(FQUERY_STRING, TmpPos + Length(VarName) + 1, MaxInt);
			{ We have to decrement the result of our search so we don't copy the
				trailing ampersand into the value we return. }
			TmpPos := Pos('&', TmpVal + '&') - 1;
			VarValue := Copy(TmpVal, 1, TmpPos);
		end;
	end;

{ The next 3 functions had their names picked up from cgic, a C library for
	writing CGI scripts.  The implementation is completely different.
	VarName -> Name of variable to look for
	RetVal -> If VarName is found, then this stores the associated value
	DefVal -> Default value if something goes wrong (invalid input, var not found)
	}
function TCGI.FormInteger(VarName: string; var RetValue: integer;
	DefValue: integer): boolean;
	var
		TmpStr: string;

	begin
		Result := GetValue(VarName, TmpStr);
		if Result then begin
			TmpStr := HTTPDecode(TmpStr);
			RetValue := StrToIntDef(TmpStr, DefValue);
		end else
			RetValue := DefValue;
	end;

function TCGI.FormFloat(VarName: string; var RetValue: double;
	DefValue: double): boolean;
	var
		TmpStr: string;

	begin
		Result := GetValue(VarName, TmpStr);
		if Result then begin
			TmpStr := HTTPDecode(TmpStr);
			try
				RetValue := StrToFloat(TmpStr);
			except
				RetValue := DefValue;
			end;
		end	else
			RetValue := DefValue;
	end;

function TCGI.FormString(VarName: string; var RetValue: string;
	DefValue: string): boolean;
	var
		TmpStr: string;

	begin
		Result := GetValue(VarName, TmpStr);
		if Result then
			RetValue := HTTPDecode(TmpStr)
		else
			RetValue := DefValue;
	end;

{ This function will write proper HTTP header to output
	Expire = 0: Page never expires
	Expire < Now: Page expires immediately }
procedure TCGI.ReturnHeader(ContentType: string; Expire: TDateTime);
	begin
		write('Content-type: ');
		if (ContentType <> '') then
			writeln(ContentType)
		else
			writeln('text/html');
		if (Expire > 0) then begin
			Expire := LocalToGMT(Expire);
			writeln(Format(FormatDateTime('"Expires: "' + DateFormat + ' "GMT"', Expire), [DayofWeekStr(Expire), MonthStr(Expire)]));
		end;
		writeln('Content-Language: en');
		// following commented line is problem for Netscape 6
		// I don't know why but who cares
		// writeln('Content-Encoding: x-compress');
		if (FCOOKIES <> '') then
			writeln(FCOOKIES);
		writeln;
	end;

{ This function will write proper HTTP header to output.
	but after this page produced will be marked as expired
	Browser will not cache this page }
procedure TCGI.ReturnHeaderExpire(ContentType: string);
	begin
		ReturnHeader(ContentType, Now);
	end;

{ Redirect browser to URL }
procedure TCGI.Redirect(URL: string);
	begin
		writeln('Location: ' + URL);
		writeln;
	end;

{ Function for debugging }
procedure TCGI.ReturnVariables;
	begin
		ReturnHeaderExpire('text/html');
		writeln('<HTML><HEAD><TITLE>Delphi CGI program</TITLE></HEAD>');
		writeln('<BODY>');
		writeln('<h1>List of environment variables</h1>');
		writeln('<br>AUTH_TYPE=' + AUTH_TYPE);
		writeln('<br>CONTENT_LENGTH=' + FCONTENT_LENGTH);
		writeln('<br>CONTENT_TYPE=' + CONTENT_TYPE);
		writeln('<br>GATEWAY_INTERFACE=' + GATEWAY_INTERFACE);
		writeln('<br>PATH_INFO=' + PATH_INFO);
		writeln('<br>PATH_TRANSLATED=' + PATH_TRANSLATED);
		writeln('<br>REMOTE_ADDR=' + REMOTE_ADDR);
		writeln('<br>REMOTE_HOST=' + REMOTE_HOST);
		writeln('<br>REMOTE_IDENT=' + REMOTE_IDENT);
		writeln('<br>REMOTE_USER=' + REMOTE_USER);
		writeln('<br>REQUEST_METHOD=' + REQUEST_METHOD);
		writeln('<br>SCRIPT_NAME=' + SCRIPT_NAME);
		writeln('<br>SERVER_NAME=' + SERVER_NAME);
		writeln('<br>SERVER_PORT=' + SERVER_PORT);
		writeln('<br>SERVER_PROTOCOL=' + SERVER_PROTOCOL);
		writeln('<br>SERVER_SOFTWARE=' + SERVER_SOFTWARE);
		writeln('<br>HTTP_ACCEPT=' + HTTP_ACCEPT);
		writeln('<br>HTTP_USER_AGENT=' + HTTP_USER_AGENT);
		writeln('<br>HTTP_REFERER=' + HTTP_REFERER);
		writeln('<br>HTTP_COOKIE=' + HTTP_COOKIE);
		writeln('<br>QUERY_STRING=' + QUERY_STRING);
		writeln('</BODY></HTML>');
	end;

{ XML function - this function will make proper node - value
	pair when NodeValue is type of string (must be XML Encoded) }
function TCGI.WriteOneNodeS(NodeName, NodeValue: string): string;
	begin
		Result := '<' + NodeName + '>' + XMLEncode(NodeValue) +
							'</' + NodeName + '>' + LN_BR;
	end;

{ XML function - this function will make proper node - value
	pair when NodeValue is not string (does not have to be XML encoded) }
function TCGI.WriteOneNode(NodeName, NodeValue: string): string;
	begin
		Result := '<' + NodeName + '>' + NodeValue +
							'</' + NodeName + '>' + LN_BR;
	end;

{ Function will return DayOfWeek as string }
function TCGI.DayOfWeekStr(DateTime: TDateTime): string;
	begin
		Result := DaysOfWeek[DayOfWeek(DateTime)];
	end;

{ Function will return Month as string }
function TCGI.MonthStr(DateTime: TDateTime): string;
	var
		Y, M, D: Word;

	begin
		DecodeDate(DateTime, Y, M, D);
		Result := Months[M];
	end;

{ This function will add new cookie to Cookie output string
	If you want to clear already set cookie from browser, then set Expire < Now }
procedure TCGI.AddCookie(CookieName: string; CookieValue: string;
			Expire: TDateTime; PathName: string; DomainName: string; Secure: boolean);
	var
		TmpStr: string;

	begin
		TmpStr := '';
		if (CookieName <> '' ) then begin
			TmpStr := Format('Set-Cookie: %s=%s', [CookieName, CookieValue]);
			if (Expire > 0) then begin
				Expire := LocalToGMT(Expire);
				TmpStr := TmpStr + Format(FormatDateTime('"; expires="' + DateFormat +
									' "GMT"', Expire), [DayOfWeekStr(Expire), MonthStr(Expire)]);
			end;
			if (PathName <> '') then
				TmpStr := TmpStr + Format('; path=%s', [PathName]);
			if (DomainName <> '') then
				TmpStr := TmpStr + Format('; domain=%s', [DomainName]);
			if Secure then
				TmpStr := TmpStr + '; secure';
		end;
		if (TmpStr <> '') then
			FCOOKIES := FCOOKIES + TmpStr + LN_BR;
	end;

{ This function resets Cookie output string
	Normally called once before setting other cookies. }
procedure TCGI.ResetCookieString;
	begin
		FCOOKIES := '';
	end;

{ Call this function if you want value from browser cookie
	See GetValue for detailed information about function, as they work
	in a very similar manner. }
function TCGI.GetCookie(CookieName: string; var CookieValue: string): boolean;
	var
		TmpPos: integer;
		TmpVal: string;

	begin
		TmpPos := Pos(' ' + CookieName + '=', ' ' + FHTTP_COOKIE);
		Result := TmpPos > 0;
		if Result then begin
			TmpVal := Copy(FHTTP_COOKIE, TmpPos + Length(CookieName) + 1, MaxInt);
			TmpPos := Pos(';', TmpVal + ';') - 1;
			CookieValue := Copy(TmpVal, 1, TmpPos);
		end;
	end;

{ This function will convert local time to GM time }
function TCGI.LocalToGMT(InTime: TDateTime): TDateTime;
	var
		TZI: TTimeZoneInformation;

	begin
		Result := InTime;
		case GetTimeZoneInformation(TZI) of
			TIME_ZONE_ID_STANDARD: Result := Result + ((TZI.Bias) / (60 * 24) );
			TIME_ZONE_ID_DAYLIGHT: Result := Result + ((TZI.Bias + TZI.StandardBias +
																			 TZI.DaylightBias) / (60 * 24) );
		end;
	end;

end.