//**********************************************************************
// version: 1.0.0.4
// Programmer: Dinko Miljak - dinmi@flashmail.com
// This class encapsulate CGI function neccesery to write normal
// cgi script
// This class is developed with intention for programmer to make small
// CGI exe - typicaly - 50 K
// Some of functions names are taken from CGIC library (FormInteger, FormString,
// FormFloat)
// Basic idea how to make this class was taken from Robert Wuttke

// Update 2000-10-30: Function LocalTime to GMT Time - important for cookie handling
// Update 2000-11-22: Ted Toraasen <ted@biip.no> -> GetValue function
// Update 2000-11-23: GetCookie function is updated
// Update 2001-03-26: Netscape 6 problem - Netscape 6 shows nothing
//                    when script is executed. Problem is ReturnHeader function
//                    Thanks to Ben Hay this problem is solved
//**********************************************************************
unit cgilib;

interface

uses Windows, SysUtils;

const
  LINE_BREAK = #10;
  DateFormat = '"%s", dd "%s" yyyy hh:mm:ss';

  DaysOfWeek: array[1..7] of string = (
    'Sun', 'Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat');
  Months: array[1..12] of string = (
    'Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun', 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec');


  // Following declarations I need because Delphi 2
  // does not have this declarations inside Windows.pas unit
  TIME_ZONE_ID_INVALID = DWORD($FFFFFFFF);
  TIME_ZONE_ID_UNKNOWN  = 0;
  TIME_ZONE_ID_STANDARD = 1;
  TIME_ZONE_ID_DAYLIGHT = 2;

type
  TCGI = class
  private
    EB : Pointer; // Environment Block Pointer

    FAUTH_TYPE: String;
    FCONTENT_LENGTH: String;
    FCONTENT_TYPE: String;
    FGATEWAY_INTERFACE: String;
    FPATH_INFO: String;
    FPATH_TRANSLATED: String;
    FREMOTE_ADDR: String;
    FREMOTE_HOST: String;
    FREMOTE_IDENT: String;
    FREMOTE_USER: String;
    FREQUEST_METHOD: String;
    FSCRIPT_NAME: String;
    FSERVER_NAME: String;
    FSERVER_PORT: String;
    FSERVER_PROTOCOL: String;
    FSERVER_SOFTWARE: String;
    FQUERY_STRING: String;
	  FHTTP_ACCEPT: String;
	  FHTTP_USER_AGENT:String;
  	FHTTP_REFERER: String;

    FHTTP_COOKIE: String; // Set of cookies returned from browser


    FCOOKIES: String; // This is newly formed COOKIE STRING
  protected
    procedure GetEnvironments; // Get the default Environment Variables
  public
    constructor Create;
    destructor  Destroy; override;

    property AUTH_TYPE         : String read FAUTH_TYPE;
    property CONTENT_LENGTH    : String read FCONTENT_LENGTH;
    property CONTENT_TYPE      : String read FCONTENT_TYPE;
    property GATEWAY_INTERFACE : String read FGATEWAY_INTERFACE;
    property PATH_INFO         : String read FPATH_INFO;
    property PATH_TRANSLATED   : String read FPATH_TRANSLATED;
    property REMOTE_ADDR       : String read FREMOTE_ADDR;
    property REMOTE_HOST       : String read FREMOTE_HOST;
    property REMOTE_IDENT      : String read FREMOTE_IDENT;
    property REMOTE_USER       : String read FREMOTE_USER;
    property REQUEST_METHOD    : String read FREQUEST_METHOD;
    property SCRIPT_NAME       : String read FSCRIPT_NAME;
    property SERVER_NAME       : String read FSERVER_NAME;
    property SERVER_PORT       : String read FSERVER_PORT;
    property SERVER_PROTOCOL   : String read FSERVER_PROTOCOL;
    property SERVER_SOFTWARE   : String read FSERVER_SOFTWARE;
    property QUERY_STRING      : String read FQUERY_STRING;
    property HTTP_ACCEPT       : String read FHTTP_ACCEPT;
    property HTTP_USER_AGENT   : String read FHTTP_USER_AGENT;
    property HTTP_REFERER      : String read FHTTP_REFERER;
    property HTTP_COOKIE       : String read FHTTP_COOKIE;

    property COOKIES           : String read FCOOKIES; // This is newly formed COOKIE STRING

    function  HTTPDecode(const AStr: String): String; // Borland routine
    function  HTTPEncode(const AStr: String): String; // Borland routine
    function  XMLEncode(var InStr: string): String;

    function  GetEnvVariable(Name : String) : String; // For extra environment variable

    // These functions return true if VariableName is found
    // FormInteger, FormFloat, FormString have default value variable
    function  GetValue(VariableName: string; var VariableValue: string): boolean;
    function  FormInteger(VarName: string; var ReturnValue: integer; DefaultValue: integer): boolean;
    function  FormFloat(VarName: string; var ReturnValue: double; DefaultValue: double): boolean;
    function  FormString(VarName: string; var ReturnValue: String; DefaultValue: String): boolean;

    // These functions return header
    procedure ReturnHeaderExpire(ContentType: string);
    procedure ReturnHeader(ContentType: string; Expire: TDateTime);
    procedure Redirect(URL: string);

    // This function return Environment variables to browser for debugging
    procedure ReturnVariables;

    // XML Functions - every function return <node>value</node>LINE_BREAK
    function  WriteOneNodeS(NodeName, NodeValue: string): string;
    function  WriteOneNode(NodeName, NodeValue: string): string;

    // These functions return Month and DayOfWeek in string format - Borland routines
    function  DayOfWeekStr(DateTime: TDateTime): string;
    function  MonthStr(DateTime: TDateTime): string;

    // This function reset CookieString (which will be sent to browser to '')
    procedure ResetCookieString;
    // This function properly forms one cookie string
    procedure AddCookie(CookieName: string; CookieValue: string;
      Expire: TDateTime; PathName: string; DomainName: string;
      Secure: boolean);
    // This function extract from exvironment variable one Cookie with the name CookieName
    // if cookie is found then function return true
    function GetCookie(CookieName: string; var CookieValue: string): boolean;

    // This function will convert Local time to GMT time
    function LocalToGMT(InTime: TDateTime): TDateTime;
  end;

implementation

constructor TCGI.Create;
begin
  EB:=GetEnvironmentStrings; // Open the environment block
  if EB=nil then begin
    writeln('Environment block error - something is very wrong.');
    Halt;
  end;

  // Get all environment variables
  GetEnvironments;
  // Open output for writing, so I can write with FileWrite function
  Rewrite(Output);
end;

destructor TCGI.Destroy;
begin
  FreeEnvironmentStrings(EB); { Close the environment block }
end;

//**********************************************************************
// Classic Borland function for HTTP Decoding
//**********************************************************************
function TCGI.HTTPDecode(const AStr: String): String;
var
  Sp, Rp, Cp: PChar;
begin
  SetLength(Result, Length(AStr));
  Sp := PChar(AStr);
  Rp := PChar(Result);
  while Sp^ <> #0 do
  begin
    if not (Sp^ in ['+','%']) then
      Rp^ := Sp^
    else
      if Sp^ = '+' then
        Rp^ := ' '
      else
      begin
        inc(Sp);
        if Sp^ = '%' then
          Rp^ := '%'
        else
        begin
          Cp := Sp;
          Inc(Sp);
          Rp^ := Chr(StrToInt(Format('$%s%s',[Cp^, Sp^])));
        end;
      end;
    Inc(Rp);
    Inc(Sp);
  end;
  SetLength(Result, Rp - PChar(Result));
end;

//**********************************************************************
// Classic Borland function for HTTP Encoding
//**********************************************************************
function TCGI.HTTPEncode(const AStr: String): String;
const
  NoConversion = ['A'..'Z','a'..'z','*','@','.','_','-',
                  '0'..'9','$','!','''','(',')'];
var
  Sp, Rp: PChar;
begin
  SetLength(Result, Length(AStr) * 3);
  Sp := PChar(AStr);
  Rp := PChar(Result);
  while Sp^ <> #0 do
  begin
    if Sp^ in NoConversion then
      Rp^ := Sp^
    else
      if Sp^ = ' ' then
        Rp^ := '+'
      else
      begin
        FormatBuf(Rp^, 3, '%%%.2x', 6, [Ord(Sp^)]);
        Inc(Rp,2);
      end;
    Inc(Rp);
    Inc(Sp);
  end;
  SetLength(Result, Rp - PChar(Result));
end;

//**********************************************************************
// My function for XML Encoding
//**********************************************************************
function TCGI.XMLEncode(var InStr: string): string;
var f: integer;
    reza: string;
const
  NoConversion = ['A'..'Z','a'..'z','*','@','.','_','-',
                  '0'..'9','$','!','(',')','[',']', ' '];
begin
  reza := '';
  for f:=1 to length(inStr) do begin
    if inStr[f] in NoConversion then begin
      reza := reza + inStr[f];
    end
    else begin
      case inStr[f] of
        '<' : reza := reza + '&lt;';
        '>' : reza := reza + '&gt;';
        '&' : reza := reza + '&amp;';
        '''': reza := reza + '&apos;';
//        '"' : reza := reza + '&qout;';
        else reza := reza + Format('&#x%2.2X;', [Ord(inStr[f])]);
      end;
    end
  end;
  result := reza;
end;

//**********************************************************************
// This function return one environment variable
//**********************************************************************
function TCGI.GetEnvVariable(Name : String) : String;
var Buffer : PChar;
    BufferSize : Integer;
    ReturnValue : String;
begin
  ReturnValue := '';
  // First find out how many bytes I need
  Buffer:=StrAlloc(1);
  BufferSize:=GetEnvironmentVariable(PCHAR(Name), Buffer, 1);
  StrDispose(Buffer);
  if BufferSize <> 0 then begin
    Buffer:=StrAlloc(BufferSize + 1);
    GetEnvironmentVariable(PCHAR(Name), Buffer, BufferSize);
    ReturnValue := String(Buffer);
    StrDispose(Buffer);
  end;
  Result := ReturnValue;
end;

//**********************************************************************
// First step is to get all environment variables
//**********************************************************************
procedure TCGI.GetEnvironments;
var i : integer;
    AChar : Char;
    TempLength: integer;
begin
  FAUTH_TYPE:=GetEnvVariable('AUTH_TYPE');
  FCONTENT_TYPE:=GetEnvVariable('CONTENT_TYPE');
  FCONTENT_LENGTH:=GetEnvVariable('CONTENT_LENGTH');
  FGATEWAY_INTERFACE:=GetEnvVariable('GATEWAY_INTERFACE');
  FPATH_INFO:=GetEnvVariable('PATH_INFO');
  FPATH_TRANSLATED:=GetEnvVariable('PATH_TRANSLATED');
  FREMOTE_ADDR:=GetEnvVariable('REMOTE_ADDR');
  FREMOTE_HOST:=GetEnvVariable('REMOTE_HOST');
  FREMOTE_IDENT:=GetEnvVariable('REMOTE_IDENT');
  FREMOTE_USER:=GetEnvVariable('REMOTE_USER');
  FREQUEST_METHOD:=GetEnvVariable('REQUEST_METHOD');
  FSCRIPT_NAME:=GetEnvVariable('SCRIPT_NAME');
  FSERVER_NAME:=GetEnvVariable('SERVER_NAME');
  FSERVER_PORT:=GetEnvVariable('SERVER_PORT');
  FSERVER_PROTOCOL:=GetEnvVariable('SERVER_PROTOCOL');
  FSERVER_SOFTWARE:=GetEnvVariable('SERVER_SOFTWARE');
  FHTTP_ACCEPT:=GetEnvVariable('HTTP_ACCEPT');
  FHTTP_USER_AGENT:=GetEnvVariable('HTTP_USER_AGENT');
  FHTTP_REFERER:=GetEnvVariable('HTTP_REFERER');
  FHTTP_COOKIE:=GetEnvVariable('HTTP_COOKIE');

  // Two cases - Get and Post method
  if CompareText(FREQUEST_METHOD,'GET')=0 then begin
    // GET method
    FQUERY_STRING:=GetEnvVariable('QUERY_STRING');
    FCONTENT_LENGTH := IntToStr(Length(FQUERY_STRING));
  end
  else if CompareText(FREQUEST_METHOD,'POST')=0 then begin
    // POST method
    TempLength := StrToIntDef(FCONTENT_LENGTH, 0);
    if TempLength <> 0 then begin
      // We have to read the exact amount of characters as there may be more
      // data on the standard input
      for i :=1 to TempLength do begin
        Read(AChar);
        FQUERY_STRING:=FQUERY_STRING + AChar;
      end;
      FQUERY_STRING := FQUERY_STRING + #0;
    end
    else FQUERY_STRING:='';
  end
  else begin
    // UNKNOWN METHOD
    FQUERY_STRING := '';
  end;
end;

//**********************************************************************
// Get one variable value
// VariableName -> variable name to look for
// VariableValue -> in this variable is saved result and returned to calling function
// Result is true if VariableName is founded
//**********************************************************************
function TCGI.GetValue(VariableName: string; var VariableValue: string): boolean;
var f: integer;
    TempPos, TempEqualPos: integer;
    count  : integer;
begin
  // Find position of variable
  result := false;
  count := 0;
  tempEqualPos := 0;
  VariableValue := '';

  // this is changed thanks to Ted Toraasen <ted@biip.no>
  // Prior version source:
  //    TempPos := Pos(VariableName + '=', FQUERY_STRING);
  TempPos := Pos('&' + VariableName + '=', '&' + FQUERY_STRING);

  if TempPos > 0 then begin
    // this is changed by me to speed this loop
    // Prior version source:
    //     for f:=TempPos to Length(FQUERY_STRING) do begin
    for f:=(TempPos + Length(VariableName)) to Length(FQUERY_STRING) do begin
      if FQUERY_STRING[f] = '&' then begin
        break;
      end
      else if FQUERY_STRING[f] = '=' then begin
        count := 1;
        tempEqualPos := f;
      end
      else begin
        if count > 0 then inc(count);
      end;
    end;
    if tempEqualPos > 0 then begin
      VariableValue := copy(FQUERY_STRING, tempEqualPos + 1, count-1);
      result := true;
    end
  end;
end;

//**********************************************************************
// This function name is stollen from cgic library which is
// C language library for writing CGI scripts
// Implementation is completely different
// VarName -> variable name to look for
// ReturnValue -> in this variable is saved result and returned to calling function
// DefaultValue -> if VarName is not found then ReturnValue is DefaultValue
//**********************************************************************
function TCGI.FormInteger(VarName: string; var ReturnValue: integer; DefaultValue: integer): boolean;
var TempString: String;
begin
  Result := false;
  ReturnValue := DefaultValue;
  if GetValue(VarName, TempString) then begin
    TempString := HTTPDecode(TempString);
    try
      ReturnValue := StrToInt(TempString);
      Result := true;
    except
    end;
  end
  else ReturnValue := DefaultValue;
end;

//**********************************************************************
// This function name is stollen from cgic library which is
// C language library for writing CGI scripts
// Implementation is completely different
// VarName -> variable name to look for
// ReturnValue -> in this variable is saved result and returned to calling function
// DefaultValue -> if VarName is not found then ReturnValue is DefaultValue
//**********************************************************************
function TCGI.FormFloat(VarName: string; var ReturnValue: double; DefaultValue: double): boolean;
var TempString: String;
begin
  Result := false;
  ReturnValue := DefaultValue;
  if GetValue(VarName, TempString) then begin
    TempString := HTTPDecode(TempString);
    try
      ReturnValue := StrToFloat(TempString);
      Result := true;
    except
    end;
  end
  else ReturnValue := DefaultValue;
end;

//**********************************************************************
// This function name is stollen from cgic library which is
// C language library for writing CGI scripts
// Implementation is completely different
// VarName -> variable name to look for
// ReturnValue -> in this variable is saved result and returned to calling function
// DefaultValue -> if VarName is not found then ReturnValue is DefaultValue
//**********************************************************************
function TCGI.FormString(VarName: string; var ReturnValue: String; DefaultValue: String): boolean;
var TempString: String;
begin
  Result := false;
  ReturnValue := DefaultValue;
  if GetValue(VarName, TempString) then begin
    ReturnValue := HTTPDecode(TempString);
    Result := true;
  end
  else ReturnValue := DefaultValue;
end;

//**********************************************************************
// This function will write proper HTTP header to output
// if you put 0 as second parameter then page will never expire
// if you want page to expire immediately after loaded then
// put some old date (for example 2 is good old date)
// in this exemple page will expire immediately after loaded
//**********************************************************************
procedure TCGI.ReturnHeader(ContentType: string; Expire: TDateTime);
var TempString: string;
begin
  if Expire > 0 then begin
    Expire := LocalToGMT(Expire);
  end;
  TempString := '';
  if Expire > 0 then
      Format(FormatDateTime('"Expires: "' + DateFormat + ' "GMT"' + LINE_BREAK, Expire),
        [DayOfWeekStr(Expire), MonthStr(Expire)]);
  TempString := TempString + 'Server: MakrosistemCGIExtension' + LINE_BREAK;
  TempString := TempString + 'Content-Language: ISO-8859-1' + LINE_BREAK;
  // following commented line is problem for Netscape 6
  // I don't know why but who cares
  // TempString := TempString + 'Content-Encoding: x-compress' + LINE_BREAK;
	TempString := TempString + Format('Content-type: %s', [ContentType]) + LINE_BREAK;
  if FCOOKIES <> '' then begin
    TempString := TempString + FCOOKIES;
  end;
	TempString := TempString + LINE_BREAK;
  FileWrite(TTextRec(Output).Handle, TempString[1], Length(TempString));
end;

//**********************************************************************
// This function will write proper HTTP header to output
// but after this page produced will be marked as expired
// Browser will not cache this page
//**********************************************************************
procedure TCGI.ReturnHeaderExpire(ContentType: string);
begin
  ReturnHeader(ContentType, 10);
end;

//**********************************************************************
// This function will redirect results to another URL
//**********************************************************************
procedure TCGI.Redirect(URL: string);
var TempString: string;
begin
  TempString := 'Location: ' + URL + LINE_BREAK + LINE_BREAK;
  FileWrite(TTextRec(Output).Handle, TempString[1], Length(TempString));
end;

//**********************************************************************
// Function for debugging
//**********************************************************************
procedure TCGI.ReturnVariables;
var TempString: string;
begin
  ReturnHeaderExpire('text/html');
  TempString := '';
  TempString := TempString + '<HTML><HEAD><TITLE>Delphi CGI program</TITLE></HEAD>'+ LINE_BREAK;
  TempString := TempString + '<BODY>'+ LINE_BREAK;
  TempString := TempString + '<h1>List of environment variables</h1>'+ LINE_BREAK;
  TempString := TempString + '<br>AUTH_TYPE=' + AUTH_TYPE+ LINE_BREAK;
  TempString := TempString + '<br>CONTENT_LENGTH=' + FCONTENT_LENGTH+ LINE_BREAK;
  TempString := TempString + '<br>CONTENT_TYPE=' + CONTENT_TYPE+ LINE_BREAK;
  TempString := TempString + '<br>GATEWAY_INTERFACE=' + GATEWAY_INTERFACE+ LINE_BREAK;
  TempString := TempString + '<br>PATH_INFO=' + PATH_INFO+ LINE_BREAK;
  TempString := TempString + '<br>PATH_TRANSLATED=' + PATH_TRANSLATED+ LINE_BREAK;
  TempString := TempString + '<br>REMOTE_ADDR=' + REMOTE_ADDR+ LINE_BREAK;
  TempString := TempString + '<br>REMOTE_HOST=' + REMOTE_HOST+ LINE_BREAK;
  TempString := TempString + '<br>REMOTE_IDENT=' + REMOTE_IDENT+ LINE_BREAK;
  TempString := TempString + '<br>REMOTE_USER=' + REMOTE_USER+ LINE_BREAK;
  TempString := TempString + '<br>REQUEST_METHOD=' + REQUEST_METHOD+ LINE_BREAK;
  TempString := TempString + '<br>SCRIPT_NAME=' + SCRIPT_NAME+ LINE_BREAK;
  TempString := TempString + '<br>SERVER_NAME=' + SERVER_NAME+ LINE_BREAK;
  TempString := TempString + '<br>SERVER_PORT=' + SERVER_PORT+ LINE_BREAK;
  TempString := TempString + '<br>SERVER_PROTOCOL=' + SERVER_PROTOCOL+ LINE_BREAK;
  TempString := TempString + '<br>SERVER_SOFTWARE=' + SERVER_SOFTWARE+ LINE_BREAK;
  TempString := TempString + '<br>HTTP_ACCEPT=' + HTTP_ACCEPT+ LINE_BREAK;
  TempString := TempString + '<br>HTTP_USER_AGENT=' + HTTP_USER_AGENT+ LINE_BREAK;
  TempString := TempString + '<br>HTTP_REFERER=' + HTTP_REFERER+ LINE_BREAK;
  TempString := TempString + '<br>HTTP_COOKIE=' + HTTP_COOKIE+ LINE_BREAK;
  TempString := TempString + '<br>QUERY_STRING=' + QUERY_STRING+ LINE_BREAK;
  TempString := TempString + '</BODY></HTML>'+ LINE_BREAK;
  FileWrite(TTextRec(Output).Handle, TempString[1], Length(TempString));
end;

//**********************************************************************
// XML function - this function will make proper node - value
// pair when NodeValue is type of string (must be XML Encoded)
//**********************************************************************
function TCGI.WriteOneNodeS(NodeName, NodeValue: string): string;
begin
  Result := '<' + NodeName + '>' +
            XMLEncode(NodeValue) + '</' + NodeName + '>' + LINE_BREAK;
end;

//**********************************************************************
// XML function - this function will make proper node - value
// pair when NodeValue is not string (does not have to be XML encoded)
//**********************************************************************
function TCGI.WriteOneNode(NodeName, NodeValue: string): string;
begin
  Result := '<' + NodeName + '>' +
            NodeValue + '</' + NodeName + '>' + LINE_BREAK;
end;

//**********************************************************************
// Function will return DayOfWeek as string
//**********************************************************************
function TCGI.DayOfWeekStr(DateTime: TDateTime): string;
begin
  Result := DaysOfWeek[DayOfWeek(DateTime)];
end;

//**********************************************************************
// Function will return Month as string
//**********************************************************************
function TCGI.MonthStr(DateTime: TDateTime): string;
var Year, Month, Day: Word;

begin
  DecodeDate(DateTime, Year, Month, Day);
  Result := Months[Month];
end;


//**********************************************************************
// This function will add new cookie to Cookie output string
// If you want to clear already set cookie from browser
// then set Expire Date to be smaller than today - you can set Expire to 10
//**********************************************************************
procedure TCGI.AddCookie(CookieName: string; CookieValue: string;
      Expire: TDateTime; PathName: string; DomainName: string; Secure: boolean);
var TempString: string;
begin
  if Expire > 0 then begin
    Expire := LocalToGMT(Expire);
  end;
  TempString := '';
  if (CookieName <> '' ) then begin
    TempString := Format('Set-Cookie: %s=%s; ', [CookieName, CookieValue]);
    if PathName <> '' then begin
      TempString := TempString + Format('path=%s ;', [PathName]);
    end;
    if DomainName <> '' then begin
      TempString := TempString + Format('domain=%s ;', [DomainName]);
    end;
    if Expire > 0 then begin
      TempString := TempString +  Format(FormatDateTime('"expires="' +
       DateFormat + ' "GMT; "', Expire), [DayOfWeekStr(Expire), MonthStr(Expire)]);
    end;
    if Secure then begin
      TempString := TempString + 'secure; ';
    end;
    SetLength(TempString, Length(TempString) -2);
  end;
  if TempString <> '' then FCOOKIES := FCOOKIES + TempString + LINE_BREAK;
end;

//**********************************************************************
// This function reset Cookie output string
// This function you will call only once and then you will
// set some other cookies
//**********************************************************************
procedure TCGI.ResetCookieString;
begin
  FCOOKIES := '';
end;

//**********************************************************************
// Call this function if you want value from browser cookie
// Function return true is cookie is found
//**********************************************************************
function TCGI.GetCookie(CookieName: string; var CookieValue: string): boolean;
var TempPos: integer;
    f: integer;
    count: integer;
    TempEqualPos: integer;
begin
  Result := false;
  count := 0;
  tempEqualPos := 0;

  TempPos := Pos(' ' + CookieName + '=', ' ' + FHTTP_COOKIE);

  if TempPos > 0 then begin
    for f:=(TempPos + Length(CookieName)) to Length(FHTTP_COOKIE) do begin
      if FHTTP_COOKIE[f] = ';' then begin
        break;
      end
      else if FHTTP_COOKIE[f] = '=' then begin
        count := 1;
        tempEqualPos := f;
      end
      else begin
        if count > 0 then inc(count);
      end;
    end;
    if tempEqualPos > 0 then begin
      CookieValue := copy(FHTTP_COOKIE, tempEqualPos + 1, count-1);
      result := true;
    end
  end;
end;

//**********************************************************************
// This function will convert local time to GM time
//**********************************************************************
function TCGI.LocalToGMT(InTime: TDateTime): TDateTime;
var TZI: TTimeZoneInformation;
begin
  case GetTimeZoneInformation(TZI) of
    TIME_ZONE_ID_UNKNOWN : result := InTime;
    TIME_ZONE_ID_STANDARD: result := InTime + ((TZI.Bias) / (60 * 24) );
    TIME_ZONE_ID_DAYLIGHT: result := InTime + ((TZI.Bias + TZI.StandardBias + TZI.DaylightBias) / (60 * 24) );
    else result := InTime;
  end;
end;

end.
